#!/usr/bin/env bash
#
# Waits until cert-manager HelmRelease is ready, and then installs a SSL.com
# ClusterIssuer with our credentials into the cert-manager namespace
#
# Usage:
#
#   ./install_sslcom_issuer.sh
set -euo pipefail

# Create secret with HMAC key
b64secret=$(echo -n "${SSL_COM_EAB_HMAC_KEY}" | base64 -w0)

# Add SSL.com ClusterIssuer
kubectl apply -n cert-manager -f - <<EOF
---
apiVersion: v1
kind: Secret
metadata:
  namespace: cert-manager
  name: sslcom-eabsecret
data:
  secret: ${b64secret}
---
apiVersion: cert-manager.io/v1
kind: ClusterIssuer
metadata:
  name: sslcom-issuer
spec:
  acme:
    # The email address used for signing up with ssl.com
    email: ${SSL_COM_EMAIL_ADDRESS}
    # The ACME server URL
    server: https://acme.ssl.com/sslcom-dv-rsa
    externalAccountBinding:
      keyID: ${SSL_COM_EAB_KID}
      keySecretRef:
        name: sslcom-eabsecret
        key: secret
    # Name of a secret used to store the ACME account private key
    privateKeySecretRef:
      name: sslcom-prod
    solvers:
      - http01:
          ingress:
            class: nginx
EOF
