(async () => {
  try {
    async function sso_login_if_needed (button_text) {
      try {
        await text(button_text).exists()
        // await console.log('Logging in...')
        await click(button_text)
        await write(sso_username, into(textBox('E-mail address')))
        await write(sso_user_pw, into(textBox('Password')))
        await click('Go!')
      } catch (error) {
        await console.error(error)
        console.log('Looks like we dont need to login, continuing')
      }
    }
    const { openBrowser, goto, textBox, into, write, click, toRightOf, below, link, press, image, waitFor, closeBrowser, screenshot } = require('taiko');
    const assert = require('assert');
    const globalTimeout = 60000
    const taikoTests = process.env.TAIKO_TESTS || 'all'
    const domain = process.env.DOMAIN
    const sso_username = process.env.SSO_USERNAME
    const sso_user_pw = process.env.SSO_USER_PW

    // https://docs.taiko.dev/api/setconfig/
    // setConfig( { observeTime: 1000});
    setConfig( {
      observeTime: 0,
      // Navigation timeout value in milliseconds for navigation after performing openTab, goto, reload, goBack, goForward, click, write, clear, press and evaluate.
      navigationTimeout: globalTimeout,
      highlightOnAction: 'true'
    });

    console.log('Executing these tests: ' + taikoTests)
    await openBrowser({
      // Optimize chrome instances for docker an parallel runs, see:
      // https://docs.taiko.dev/taiko_in_docker/
      // https://docs.taiko.dev/frequently_asked_questions/#how-can-i-optimize-chrome-instances-for-parallel-runs%3F
      args: [
        "--disable-gpu",
        "--disable-dev-shm-usage",
        "--disable-setuid-sandbox",
        "--no-first-run",
        "--no-sandbox",
        "--no-zygote",
        "--start-maximized"
      ]
    });

    // Nextcloud and Onlyoffice
    if (taikoTests.includes('nextcloud') || taikoTests === 'all') {
      const nextcloudUrl = 'https://files.' + domain
      const onlyofficeUrl = 'https://office.' + domain
      console.log('• Onlyoffice')

      await goto(onlyofficeUrl + '/welcome')
      await waitFor(async () => (await text("Thank you for choosing ONLYOFFICE!").isVisible()))
      await waitFor(async () => (await text("Document Server is running").isVisible()))

      await goto(onlyofficeUrl + '/healthcheck')
      await waitFor(async () => (await text("true").isVisible()))

      console.log('• Nextcloud')
      await goto(nextcloudUrl)
      await sso_login_if_needed('Log in with Stackspin')
      await waitFor(async () => (await text("Set location for weather").isVisible()), globalTimeout)
      // Close potential nextcloud first run wizard modal
      // https://github.com/nextcloud/firstrunwizard/issues/488
      // Unfortunately, we need to sleep a while since I haven't found a
      // good way that closes the modal *if* it pops up, since these
      // tests should also work on subsequent logins.
      await waitFor(5000)
      await press('Escape')

      // Test password app
      await click('Passwords')
      await waitFor(async () => (await text("Handbook").isVisible()), globalTimeout)

      // Test if calendar app is enabled
      await click('Calendar')
      await waitFor(async () => (await text("Settings & Import").isVisible()), globalTimeout)

      console.log('• Nextcloud Onlyoffice integration')
      // Open document and type some text
      await click('Files')
      // Force page reload because of random empty pages for files app
      // https://open.greenhost.net/stackspin/nextcloud/-/issues/973
      await reload()

      await waitFor(async () => (await link({class:'new'}).isVisible()), globalTimeout)
      await click(link({class:'new'}))
      await click('Document')

      // Create random file name because NC20 will complain if file already
      // exists
      const file_rand = 'test-' + Math.random().toString(16).substr(2, 12)
      await press([...file_rand, 'Enter'])

      let italicButtonId = '#id-toolbar-btn-italic'
      await waitFor(async () => (await $(italicButtonId).isVisible()), globalTimeout)

      // Activate italic button
      let buttonStateBefore = await evaluate($(italicButtonId), (elem) => {return elem.getAttribute('class')})
      await assert.ok(!buttonStateBefore.includes('active'))

      await waitFor(async () => (await $('#id_target_cursor').isVisible()), globalTimeout)
      await click($(italicButtonId))

      let buttonStateAfter = await evaluate($(italicButtonId), (elem) => {return elem.getAttribute('class')})
      await assert.ok(buttonStateAfter.includes('active'))

      await press([...'Hi from taiko!', 'Enter'])

      // Deactivate italic finially
      await click($(italicButtonId))
      await closeTab()
    }

    // Wordpress
    if (taikoTests.includes('wordpress') || taikoTests === 'all') {
      const wordpressUrl = 'https://www.' + domain

      console.log('• Wordpress')
      await goto(wordpressUrl)
      await waitFor('Hello world!')
      await goto(wordpressUrl + '/wp-admin/')
      await sso_login_if_needed('Login with OpenID Connect')
      await assert.ok(await link('Dashboard').exists())
    }

    // Grafana
    if (taikoTests.includes('grafana') || taikoTests === 'all') {
      const grafanaUrl = 'https://grafana.' + domain

      console.log('• Grafana')
      await goto(grafanaUrl)
      await sso_login_if_needed('Sign in with Stackspin')

      // Node exporter dashboard
      // Couldn't select "Manage dashboards" from the sidebar menu easily,
      // so we just go there
      await goto(grafanaUrl + '/dashboards')
      await click(link('Nodes'))
      await('CPU Usage')

      // Explore Loki log messages
      // Currently disabled until admin user gets proper admin privileges
      // See https://open.greenhost.net/stackspin/single-sign-on/-/issues/122
      // await goto(grafanaUrl + '/explore')
      // await click(image(toRightOf('Explore')))
      // await click('Loki')
      // await click('Log browser')
      // await click('app')
      // await click('cert-manager')
      // await click('grafana')
      // await click('Show logs')
    }

    // Wekan
    if (taikoTests.includes('wekan') || taikoTests === 'all') {
      const wekanUrl = 'https://wekan.' + domain

      console.log('• Wekan')
      await goto(wekanUrl)
      await sso_login_if_needed('sign in with Oidc')
      console.log(`\nPlease note that above error message is acceptable since wei
did not find a way with taiko to handle OIDC login popup windows well.
Wekan still doesnt support OIDC redirects (https://github.com/wekan/wekan/issues/3845).\n`)
      await goto(wekanUrl)
      await assert.ok(await text('All boards').exists())
    }

    // Dashboard
    if (taikoTests.includes('dashboard') || taikoTests === 'all') {
      const dashboardUrl = 'https://dashboard.' + domain
      console.log('• Dashboard at ' + dashboardUrl)
      await goto(dashboardUrl)
    }

    // Zulip
    if (taikoTests.includes('zulip') || taikoTests === 'all') {
      const zulipUrl = 'https://zulip.' + domain

      console.log('• Zulip')
      await goto(zulipUrl)
      await sso_login_if_needed("Log in with Stackspin")
      await assert.ok(await text('Private messages').exists())
    }

  } catch (error) {
    await screenshot()
    console.error(error)
    process.exitCode = 1
  } finally {
    await closeBrowser()
  }
})()
